@@ -1,4 +1,9 @@
-# Copyright (c) 2012 Intel, Inc.
+# -*- coding: utf-8 -*-
+# vim: ts=4 sw=4 et ai si
+#
+# Copyright (c) 2012-2014 Intel, Inc.
+# License: GPLv2
+# Author: Artem Bityutskiy <artem.bityutskiy@linux.intel.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2,
@@ -9,10 +14,16 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 # General Public License for more details.
 
+import os
+import struct
+from fcntl import ioctl
+
+
 """
 This module contains various shared helper functions.
 """
 
+
 def human_size(size):
     """Transform size in bytes into a human-readable form."""
     if size == 1:
@@ -28,6 +39,7 @@ def human_size(size):
 
     return "%.1f %s" % (size, 'EiB')
 
+
 def human_time(seconds):
     """Transform time in seconds to the HH:MM:SS format."""
     (minutes, seconds) = divmod(seconds, 60)
@@ -41,19 +53,25 @@ def human_time(seconds):
 
     return result + "%.1fs" % seconds
 
+
 def get_block_size(file_obj):
     """
-    Returns block size for file object 'file_obj'. Errors are indicated by the
+    Return block size for file object 'file_obj'. Errors are indicated by the
     'IOError' exception.
     """
 
-    from fcntl import ioctl
-    import struct
-
     # Get the block size of the host file-system for the image file by calling
     # the FIGETBSZ ioctl (number 2).
     binary_data = ioctl(file_obj, 2, struct.pack('I', 0))
-    return struct.unpack('I', binary_data)[0]
+    bsize = struct.unpack('I', binary_data)[0]
+    if not bsize:
+        stat = os.fstat(file_obj.fileno())
+        if hasattr(stat, 'st_blksize'):
+            bsize = stat.st_blksize
+        else:
+            raise IOError("Unable to determine block size")
+    return bsize
+
 
 def program_is_available(name):
     """
@@ -61,8 +79,6 @@ def program_is_available(name):
     available in the system.
     """
 
-    import os
-
     for path in os.environ["PATH"].split(os.pathsep):
         program = os.path.join(path.strip('"'), name)
         if os.path.isfile(program) and os.access(program, os.X_OK):
