@@ -1,4 +1,9 @@
-# Copyright (c) 2012-2013 Intel, Inc.
+# -*- coding: utf-8 -*-
+# vim: ts=4 sw=4 et ai si
+#
+# Copyright (c) 2012-2014 Intel, Inc.
+# License: GPLv2
+# Author: Artem Bityutskiy <artem.bityutskiy@linux.intel.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License, version 2,
@@ -14,19 +19,25 @@ This module allows opening and reading local and remote files and decompress
 them on-the-fly if needed. Remote files are read using urllib2 (except of
 "ssh://" URLs, which are handled differently). Supported file extentions are:
 'bz2', 'gz', 'xz', 'lzo' and a "tar" version of them: 'tar.bz2', 'tbz2', 'tbz',
-'tb2', 'tar.gz', 'tgz', 'tar.xz', 'txz', 'tar.lzo', 'tzo'. This module uses
-the following system programs for decompressing: pbzip2, bzip2, gzip, pigz, xz,
-lzop, and tar.
+'tb2', 'tar.gz', 'tgz', 'tar.xz', 'txz', 'tar.lzo', 'tzo', 'tar.lz4', 'tlz4'.
+This module uses the following system programs for decompressing: pbzip2, bzip2,
+gzip, pigz, xz, lzop, lz4, tar and unzip.
 """
 
 import os
 import errno
-import urlparse
+import sys
+if sys.version[0] == '2':
+    import urlparse
+else:
+    import urllib.parse as urlparse
 import logging
 import threading
 import subprocess
 from bmaptools import BmapHelpers
 
+_log = logging.getLogger(__name__)  # pylint: disable=C0103
+
 # Disable the following pylint errors and recommendations:
 #   * Instance of X has no member Y (E1101), because it produces
 #     false-positives for many of 'subprocess' class members, e.g.
@@ -42,8 +53,10 @@ from bmaptools import BmapHelpers
 # pylint: disable=R0915
 
 # A list of supported compression types
-SUPPORTED_COMPRESSION_TYPES = ('bz2', 'gz', 'xz', 'lzo', 'tar.gz', 'tar.bz2',
-                               'tar.xz', 'tar.lzo')
+SUPPORTED_COMPRESSION_TYPES = ('bz2', 'gz', 'xz', 'lzo', 'lz4', 'tar.gz',
+                               'tar.bz2', 'tar.xz', 'tar.lzo', 'tar.lz4',
+                               'zip')
+
 
 def _fake_seek_forward(file_obj, cur_pos, offset, whence=os.SEEK_SET):
     """
@@ -81,6 +94,7 @@ def _fake_seek_forward(file_obj, cur_pos, offset, whence=os.SEEK_SET):
 
     return new_pos - to_read
 
+
 class Error(Exception):
     """
     A class for exceptions generated by this module. We currently support only
@@ -89,6 +103,7 @@ class Error(Exception):
     """
     pass
 
+
 def _decode_sshpass_exit_code(code):
     """
     A helper function which converts "sshpass" command-line tool's exit code
@@ -116,23 +131,19 @@ def _decode_sshpass_exit_code(code):
 
     return result
 
+
 class TransRead(object):
     """
     This class implement the transparent reading functionality. Instances of
     this class are file-like objects which you can read and seek only forward.
     """
 
-    def __init__(self, filepath, log=None):
+    def __init__(self, filepath):
         """
         Class constructor. The 'filepath' argument is the full path to the file
-        to read transparently. The "log" argument is the logger object to use
-        for printing messages.
+        to read transparently.
         """
 
-        self._log = log
-        if self._log is None:
-            self._log = logging.getLogger(__name__)
-
         self.name = filepath
         # Size of the file (in uncompressed form), may be 'None' if the size is
         # unknown
@@ -195,7 +206,7 @@ class TransRead(object):
         object, while 'f_to' is usually stdin of the decompressor process.
         """
 
-        chunk_size = 1024*1024
+        chunk_size = 1024 * 1024
         while not self._done:
             buf = f_from.read(chunk_size)
             if not buf:
@@ -239,6 +250,12 @@ class TransRead(object):
                 return True
             return False
 
+        def is_lz4(name):
+            """Returns 'True' if file 'name' is compressed with 'lz4'."""
+            if name.endswith('.lz4') and not name.endswith('.tar.lz4'):
+                return True
+            return False
+
         def is_tar_gz(name):
             """
             Returns 'True' if file 'name' is a tar archive compressed with
@@ -279,6 +296,16 @@ class TransRead(object):
                 return True
             return False
 
+        def is_tar_lz4(name):
+            """
+            Returns 'True' if file 'name' is a tar archive compressed with
+            'lz4'.
+            """
+
+            if name.endswith('.tar.lz4') or name.endswith('.tlz4'):
+                return True
+            return False
+
         archiver = None
         if is_tar_gz(self.name) or is_gzip(self.name):
             self.compression_type = 'gzip'
@@ -320,6 +347,18 @@ class TransRead(object):
             else:
                 archiver = "tar"
                 args = "-x --lzo -O"
+        elif self.name.endswith(".zip"):
+            self.compression_type = 'zip'
+            decompressor = "funzip"
+            args = ""
+        elif is_tar_lz4(self.name) or is_lz4(self.name):
+            self.compression_type = 'lz4'
+            decompressor = "lz4"
+            if is_lz4(self.name):
+                args = "-d -c"
+            else:
+                archiver = "tar"
+                args = "-x -Ilz4 -O"
         else:
             if not self.is_url:
                 self.size = os.fstat(self._f_objs[-1].fileno()).st_size
@@ -340,16 +379,23 @@ class TransRead(object):
             args = archiver + " " + args
         else:
             args = decompressor + " " + args
+
+        if hasattr(self._f_objs[-1], 'fileno'):
+            child_stdin = self._f_objs[-1].fileno()
+        else:
+            child_stdin = subprocess.PIPE
+
         child_process = subprocess.Popen(args, shell=True,
-                                         bufsize=1024*1024,
-                                         stdin=subprocess.PIPE,
+                                         bufsize=1024 * 1024,
+                                         stdin=child_stdin,
                                          stdout=subprocess.PIPE,
                                          stderr=subprocess.PIPE)
 
-        args = (self._f_objs[-1], child_process.stdin, )
-        self._rthread = threading.Thread(target=self._read_thread, args=args)
-        self._rthread.daemon = True
-        self._rthread.start()
+        if child_stdin == subprocess.PIPE:
+            args = (self._f_objs[-1], child_process.stdin, )
+            self._rthread = threading.Thread(target=self._read_thread, args=args)
+            self._rthread.daemon = True
+            self._rthread.start()
 
         self._force_fake_seek = True
         self._f_objs.append(child_process.stdout)
@@ -411,7 +457,7 @@ class TransRead(object):
         # host
         command = "test -f " + path + " && test -r " + path
         child_process = subprocess.Popen(popen_args + [command],
-                                         bufsize=1024*1024,
+                                         bufsize=1024 * 1024,
                                          stdout=subprocess.PIPE)
         child_process.wait()
         if child_process.returncode != 0:
@@ -436,19 +482,24 @@ class TransRead(object):
         Open an URL 'url' and return the file-like object of the opened URL.
         """
 
-        def _print_warning(log, timeout):
+        def _print_warning(timeout):
             """
             This is a small helper function for printing a warning if we cannot
             open the URL for some time.
             """
-            log.warning("failed to open the URL with %d sec timeout, is the "
-                        "proxy configured correctly? Keep trying ..." %
-                        timeout)
+            _log.warning("failed to open the URL with %d sec timeout, is the "
+                         "proxy configured correctly? Keep trying ..." %
+                         timeout)
 
-        import urllib2
-        import httplib
         import socket
 
+        if sys.version[0] == '2':
+            import httplib
+            import urllib2
+        else:
+            import http.client as httplib
+            import urllib.request as urllib2
+
         parsed_url = urlparse.urlparse(url)
 
         if parsed_url.scheme == "ssh":
@@ -491,18 +542,18 @@ class TransRead(object):
         # about something possibly being wrong.
         for timeout in (10, None):
             try:
-                f_obj = opener.open(url, timeout = timeout)
+                f_obj = opener.open(url, timeout=timeout)
             # Handling the timeout case in Python 2.7
-            except socket.timeout, err:
+            except socket.timeout as err:
                 if timeout is not None:
-                    _print_warning(self._log, timeout)
+                    _print_warning(timeout)
                 else:
                     raise Error("cannot open URL '%s': %s" % (url, err))
             except urllib2.URLError as err:
                 # Handling the timeout case in Python 2.6
                 if timeout is not None and \
                    isinstance(err.reason, socket.timeout):
-                    _print_warning(self._log, timeout)
+                    _print_warning(timeout)
                 else:
                     raise Error("cannot open URL '%s': %s" % (url, err))
             except (IOError, ValueError, httplib.InvalidURL) as err:
